import 'dart:async';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:global/global.dart';
import 'package:global/src/GFilePaths.dart';

class GUtils {
  //Utility function for getting RoundBox decoration
  static BoxDecoration getRoundBoxDecoration({
    double radius = 6,
    double borderWidth = 0.75,
    Color fillColor = Colors.white,
    Color borderColor = const Color(0xFFD4D4D4),
    BorderRadiusGeometry? borderRadiusGeometry,
  }) {
    BorderSide borderSide = BorderSide(
      color: borderColor,
      width: borderWidth,
      style: borderWidth != 0 ? BorderStyle.solid : BorderStyle.none,
    );
    return BoxDecoration(
      color: fillColor,
      border: Border(
          top: borderSide,
          left: borderSide,
          bottom: borderSide,
          right: borderSide),
      borderRadius:
          borderRadiusGeometry ?? BorderRadius.all(Radius.circular(radius)),
    );
  }

  static OutlineInputBorder getRoundOutlineInputBorder({
    Color color = const Color(0xFFD4D4D4),
    double radius = 6,
    double width = 0.75,
  }) =>
      OutlineInputBorder(
        borderRadius: BorderRadius.all(Radius.circular(radius)),
        borderSide: BorderSide(
          color: color,
          width: width,
          style: width == 0 ? BorderStyle.none : BorderStyle.solid,
        ),
      );

  static RoundedRectangleBorder getRoundedRectBorder({
    double radius = 6,
    double width = 0.75,
    Color color = const Color(0xFFD4D4D4),
  }) =>
      RoundedRectangleBorder(
        side: BorderSide(
            width: width,
            color: color,
            style: width == 0 ? BorderStyle.none : BorderStyle.solid),
        borderRadius: BorderRadius.circular(radius),
      );

  static Widget getFullScreenWidget({
    required BuildContext context,
    required Widget child,
    Widget? floatingActionButton,
  }) =>
      Scaffold(
        body: SafeArea(
          child: Container(
            height: getAvlblScreenHeight(context),
            color: Colors.white,
            child: child,
          ),
        ),
        floatingActionButton: floatingActionButton,
      );

  static Widget getElevatedButton({
    required Widget content,
    required Function() onPressed,
    Color? backgroundColor,
    Color? overlayColor,
    double leftRightPadding = 12,
    double topBottomPadding = 10,
    OutlinedBorder? border,
    int flex = 1,
  }) =>
      Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          Expanded(
            flex: flex,
            child: ElevatedButton(
              onPressed: onPressed,
              style: ButtonStyle(
                  minimumSize: MaterialStateProperty.all(Size.zero),
                  tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                  padding: MaterialStateProperty.all(EdgeInsets.fromLTRB(
                      leftRightPadding,
                      topBottomPadding,
                      leftRightPadding,
                      topBottomPadding)),
                  backgroundColor: MaterialStateColor.resolveWith(
                      (states) => backgroundColor ?? Global.themeColor),
                  overlayColor: MaterialStateColor.resolveWith((states) =>
                      overlayColor ??
                      darken(backgroundColor ?? Global.themeColor, 0.1)),
                  shape: backgroundColor == Colors.white
                      ? MaterialStateProperty.all(
                          border ?? getRoundedRectBorder())
                      : null),
              child: Column(
                children: [content],
              ),
            ),
          ),
        ],
      );

  //Utility function for giving a darkened color
  static Color darken(Color color, [double amount = .1]) {
    assert(amount >= 0 && amount <= 1);

    final hsl = HSLColor.fromColor(color);
    final hslDark = hsl.withLightness((hsl.lightness - amount).clamp(0.0, 1.0));

    return hslDark.toColor();
  }

  //Utility function for giving a lightened color
  static Color lighten(Color color, [double amount = .1]) {
    assert(amount >= 0 && amount <= 1);

    final hsl = HSLColor.fromColor(color);
    final hslLight =
        hsl.withLightness((hsl.lightness + amount).clamp(0.0, 1.0));

    return hslLight.toColor();
  }

  //Utility function for showing a toast
  static Future<bool?> showToast(
          {required String message,
          required Toast toastWithDuration,
          Color textColor = Colors.white,
          Color backgroundColor = const Color(0xFF838383),
          ToastGravity gravity = ToastGravity.BOTTOM}) =>
      Fluttertoast.showToast(
        msg: message,
        toastLength: toastWithDuration,
        gravity: gravity,
        fontSize: 14,
        textColor: textColor,
        backgroundColor: backgroundColor,
      );

  //Utility function for showing Progress Dialog
  static Future<dynamic> showProgressDialog() => showMyDialog(
        dialogBody: getProgressDialogWidget(),
        canPop: false,
      );

  static Widget getProgressDialogWidget({bool isForNewRoute = true}) =>
      Material(
        type: MaterialType.transparency,
        child: SizedBox(
          width: double.infinity,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.max,
            children: <Widget>[
              CircularProgressIndicator(
                valueColor: AlwaysStoppedAnimation<Color>(
                    isForNewRoute ? Colors.white : const Color(0xFF4D4D4D)),
              )
            ],
          ),
        ),
      );

  static Future<T?> showMyDialog<T>({
    required Widget dialogBody,
    bool canPop = true,
    bool isAlertDialog = false,
  }) =>
      showDialog(
        context: Global.navigatorKey.currentState!.context,
        builder: (dialogContext) => PopScope(
          canPop: canPop,
          child: StatefulBuilder(
            builder: (context, setState) => Material(
              type: MaterialType.transparency,
              child: Padding(
                  padding: MediaQuery.of(dialogContext).viewInsets,
                  child: Align(
                    alignment: Alignment.center,
                    child: Container(
                      padding: EdgeInsets.only(
                          left: isAlertDialog ? 40 : 20,
                          right: isAlertDialog ? 40 : 20),
                      child: dialogBody,
                    ),
                  )),
            ),
          ),
        ),
      );

  static Function onDownloadConfirmed({
    required String url,
    required Function onPermanentlyDenied,
  }) =>
      () async {
        bool isStoragePermGranted;
        if (Platform.isIOS ||
            (await DeviceInfoPlugin().androidInfo).version.sdkInt > 28) {
          isStoragePermGranted = true;
        } else {
          switch (await Permission.storage.status) {
            case PermissionStatus.granted:
              isStoragePermGranted = true;
              break;

            default:
              /*We are handling only [PermissionStatus.granted] and [default] case,
               because 'Permission.storage.status' always returns either
               [PermissionStatus.granted] or [PermissionStatus.denied] status*/
              switch (await Permission.storage.request()) {
                case PermissionStatus.granted:
                  isStoragePermGranted = true;
                  break;

                case PermissionStatus.denied:
                  isStoragePermGranted = false;
                  break;

                case PermissionStatus.permanentlyDenied:
                  isStoragePermGranted = false;
                  onPermanentlyDenied();
                  break;

                default:
                  isStoragePermGranted = false;
                  break;
              }
              break;
          }
        }
        if (isStoragePermGranted) {
          String localStoragePath;
          if (Platform.isAndroid) {
            try {
              localStoragePath = await AndroidPathProvider.downloadsPath;
            } catch (_, __) {
              localStoragePath = (await getExternalStorageDirectory())!.path;
            }
          } else {
            localStoragePath =
                (await getApplicationDocumentsDirectory()).absolute.path;
          }
          final savedDir = Directory(localStoragePath);
          if (!savedDir.existsSync()) {
            await savedDir.create();
          }
          FlutterDownloader.enqueue(
            url: url,
            saveInPublicStorage: true,
            savedDir: localStoragePath,
          );
        }
      };

  static Function onGoToAppSettingsConfirmed() =>
      () => openAppSettings().then((couldOpen) {
            if (!couldOpen) {
              GUtils.showToast(
                  message: GTexts.appSettingsPageCouldNotBeOpened,
                  toastWithDuration: Toast.LENGTH_LONG);
            }
          });

  //Utility function for getting available screen height in a device
  static double getAvlblScreenHeight(BuildContext context) =>
      MediaQuery.of(context).size.height -
      MediaQuery.of(context).padding.top -
      MediaQuery.of(context).padding.bottom;

  static printInDebug(Object object) {
    if (kDebugMode) {
      print(object);
    }
  }

  static List<Shadow> getIconShadows() => [
        const Shadow(
            color: Colors.grey, blurRadius: 2.0, offset: Offset(0, 0.8))
      ];

  static String toCamelCase(String source) =>
      '${source[0].toUpperCase()}${source.substring(1).toLowerCase()}';

  static Widget getTextFormField({
    required TextEditingController controller,
    String? hintText,
    String? labelText,
    bool? enabled,
    String? Function(String?)? validator,
    FocusNode? focusNode,
    AutovalidateMode? autovalidateMode,
    TextInputType? keyboardType,
    int? maxLength,
    String? counterText,
    int? maxLines = 1,
    List<TextInputFormatter>? inputFormatters,
    TextStyle? style,
    bool obscureText = false,
    Color fillColor = Colors.white,
    TextStyle? floatingLabelStyle,
    TextStyle? labelStyle,
    EdgeInsetsGeometry? contentPadding = const EdgeInsets.all(10),
    InputBorder? border,
    InputBorder? enabledBorder,
    InputBorder? disabledBorder,
    InputBorder? focusedBorder,
    InputBorder? errorBorder,
    InputBorder? focusedErrorBorder,
    Widget? prefixIcon,
    Widget? suffixIcon,
    ValueChanged<String>? onChanged,
    Future<dynamic> Function()? onTap,
    FutureOr<dynamic> Function(dynamic)? onResult,
  }) =>
      TextFormField(
        enabled: enabled,
        enableInteractiveSelection: onTap == null ? null : false,
        readOnly: onTap != null,
        autovalidateMode: autovalidateMode,
        focusNode: focusNode,
        controller: controller,
        validator: validator,
        obscureText: obscureText,
        inputFormatters: inputFormatters,
        style: style ??
            (enabled == null || enabled
                ? Global.defTFFEnabledStyle
                : Global.defTFFDisabledStyle),
        keyboardType: keyboardType,
        maxLines: maxLines,
        maxLength: maxLength,
        decoration: InputDecoration(
          filled: true,
          fillColor: fillColor,
          prefixIcon: prefixIcon,
          suffixIcon: suffixIcon,
          hintText: hintText,
          labelText: labelText,
          contentPadding: contentPadding,
          counterText: counterText,
          errorMaxLines: 2,
          floatingLabelStyle:
              floatingLabelStyle ?? Global.defTFFFloatingLabelStyle,
          hintStyle: labelStyle ?? Global.defTFFHintOrLabelStyle,
          labelStyle: labelStyle ?? Global.defTFFHintOrLabelStyle,
          border: border ?? GUtils.getRoundOutlineInputBorder(),
          enabledBorder: enabledBorder ?? GUtils.getRoundOutlineInputBorder(),
          disabledBorder: disabledBorder ?? GUtils.getRoundOutlineInputBorder(),
          focusedBorder: focusedBorder ??
              GUtils.getRoundOutlineInputBorder(color: const Color(0xFF757575)),
          errorBorder: errorBorder ??
              GUtils.getRoundOutlineInputBorder(
                  color: const Color(0xFFE83333), width: 1),
          focusedErrorBorder: focusedErrorBorder ??
              GUtils.getRoundOutlineInputBorder(
                  color: const Color(0xFFE83333), width: 1.2),
          errorStyle: const TextStyle(height: 0),
        ),
        onChanged: onChanged,
        onTap: onTap == null
            ? null
            : () async {
                var result = await onTap();
                if ((result == null && controller.text.trim().isEmpty) ||
                    (result is List && result.isEmpty)) {
                  FocusScope.of(Global.navigatorKey.currentState!.context)
                      .requestFocus(FocusNode());
                }
                onResult!(result);
              },
      );

  static Widget getSearchDateSelector({
    bool? enabled,
    required String headerText,
    required DateTime Function() getFirstDate,
    required DateTime? Function() getInitialDate,
    required DateTime Function() getLastDate,
    required TextEditingController controller,
    Color fillColor = Colors.white,
    TextStyle? floatingLabelStyle,
    TextStyle? labelStyle,
    EdgeInsetsGeometry? contentPadding = const EdgeInsets.all(10),
    InputBorder? border,
    InputBorder? enabledBorder,
    InputBorder? disabledBorder,
    InputBorder? focusedBorder,
    InputBorder? errorBorder,
    InputBorder? focusedErrorBorder,
    Widget? suffixIcon,
    String? Function(String?)? validator,
    AutovalidateMode? autoValidateMode,
  }) =>
      getTextFormField(
        labelText: headerText,
        enabled: enabled,
        controller: controller,
        fillColor: fillColor,
        floatingLabelStyle: floatingLabelStyle,
        labelStyle: labelStyle,
        contentPadding: contentPadding,
        border: border,
        enabledBorder: enabledBorder,
        disabledBorder: disabledBorder,
        focusedBorder: focusedBorder,
        errorBorder: errorBorder,
        focusedErrorBorder: focusedErrorBorder,
        onTap: () => showDatePicker(
          context: Global.navigatorKey.currentState!.context,
          firstDate: getFirstDate(),
          initialDate: controller.text.isEmpty
              ? getInitialDate()
              : GDateFormats.MMSlashddSlashyyyy.parse(controller.text),
          lastDate: getLastDate(),
          builder: (context, child) => Theme(
            data: Theme.of(context).copyWith(
              colorScheme: ColorScheme.light(primary: Global.themeColor),
            ),
            child: child!,
          ),
        ),
        onResult: (result) {
          if (result != null) {
            controller.text = GDateFormats.MMSlashddSlashyyyy.format(result);
          }
        },
        suffixIcon: suffixIcon ??
            Align(
              widthFactor: 0.5,
              heightFactor: 0.5,
              child: SvgPicture.asset(
                GFilePaths.dateInCalendar,
                colorFilter:
                    ColorFilter.mode(Global.themeColor, BlendMode.srcIn),
                width: 20,
                height: 20,
              ),
            ),
        validator: validator ?? GValidators.emptyStringValidator,
        autovalidateMode: autoValidateMode,
      );

  static Widget getDropdownButtonFormField<T>({
    required String labelText,
    required List<DropdownMenuItem<T>> items,
    required T? value,
    required ValueChanged<T?>? onChanged,
    Color fillColor = Colors.white,
    TextStyle? floatingLabelStyle,
    TextStyle? labelStyle,
    EdgeInsetsGeometry? contentPadding = const EdgeInsets.all(10),
    InputBorder? border,
    InputBorder? enabledBorder,
    InputBorder? disabledBorder,
    InputBorder? focusedBorder,
    InputBorder? errorBorder,
    InputBorder? focusedErrorBorder,
    String? Function(T?)? validator,
    AutovalidateMode? autovalidateMode,
    Widget? hint,
  }) =>
      DropdownButtonFormField<T>(
        isExpanded: true,
        decoration: InputDecoration(
          filled: true,
          fillColor: fillColor,
          contentPadding: contentPadding,
          labelText: labelText,
          floatingLabelBehavior: FloatingLabelBehavior.always,
          floatingLabelStyle:
              floatingLabelStyle ?? Global.defTFFFloatingLabelStyle,
          border: border ?? GUtils.getRoundOutlineInputBorder(),
          enabledBorder: enabledBorder ?? GUtils.getRoundOutlineInputBorder(),
          disabledBorder: disabledBorder ?? GUtils.getRoundOutlineInputBorder(),
          focusedBorder: focusedBorder ??
              GUtils.getRoundOutlineInputBorder(color: const Color(0xFF757575)),
          errorBorder: errorBorder ??
              GUtils.getRoundOutlineInputBorder(
                  color: const Color(0xFFE83333), width: 1),
          focusedErrorBorder: focusedErrorBorder ??
              GUtils.getRoundOutlineInputBorder(
                  color: const Color(0xFFE83333), width: 1.2),
          errorStyle: const TextStyle(height: 0),
        ),
        hint: hint,
        value: value,
        items: items,
        onChanged: onChanged,
        validator: validator,
        autovalidateMode: autovalidateMode,
      );
}
